@@ -1,7 +1,7 @@
 /*
  *  Authors: Rodney Dawes <dobey@ximian.com>
  *
- *  Copyright 2003 Novell, Inc. (www.novell.com)
+ *  Copyright 2003-2004 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of version 2 of the GNU General Public License
@@ -74,87 +74,47 @@ static void wp_properties_help (GtkWindow * parent, char const * helpfile,
   }
 }
 
-static void gnome_wp_add_image (GnomeWPCapplet * capplet,
-				const gchar * filename) {
-  GnomeWPItem * item;
+static void gnome_wp_capplet_scroll_to_item (GnomeWPCapplet * capplet,
+					     GnomeWPItem * item) {
   GtkTreePath * path;
-  GdkColor color1, color2;
+
+  g_return_if_fail (capplet != NULL);
+  g_return_if_fail (item != NULL);
+
+  path = gtk_tree_row_reference_get_path (item->rowref);
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
+			    NULL, FALSE);
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
+				path, NULL, TRUE, 0.5, 0.0);
+  gtk_tree_path_free (path);
+}
+
+static GnomeWPItem * gnome_wp_add_image (GnomeWPCapplet * capplet,
+					 const gchar * filename) {
+  GnomeWPItem * item;
 
   item = g_hash_table_lookup (capplet->wphash, filename);
   if (item != NULL) {
-    GtkTreePath * path;
-    
     if (item->deleted) {
       item->deleted = FALSE;
       wp_props_load_wallpaper (item->filename, item, capplet);
     }
-
-    path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-			      NULL, FALSE);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				  path, NULL, TRUE, 0.5, 0.0);
-    gtk_tree_path_free (path);
-
-    return;
-  }
-
-  item = g_new0 (GnomeWPItem, 1);
-  
-  item->filename = g_strdup (filename);
-
-  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-
-  item->shade_type = gconf_client_get_string (capplet->client,
-					      WP_SHADING_KEY, NULL);
-  item->pri_color = gconf_client_get_string (capplet->client,
-					     WP_PCOLOR_KEY, NULL);
-  item->sec_color = gconf_client_get_string (capplet->client,
-					     WP_SCOLOR_KEY, NULL);
-
-  gdk_color_parse (item->pri_color, &color1);
-  gdk_color_parse (item->sec_color, &color2);
-     
-  item->pcolor = gdk_color_copy (&color1);
-  item->scolor = gdk_color_copy (&color2);
-
-  if (!strncmp (item->fileinfo->mime_type, "image/", strlen ("image/"))) {
-    if (item->name == NULL) {
-      item->name = g_strdup (item->fileinfo->name);
-    }
-    item->options = gconf_client_get_string (capplet->client,
-					     WP_OPTIONS_KEY,
-					     NULL);
-    if (!strcmp (item->options, "none")) {
-      item->options = g_strdup ("wallpaper");
-    }
-    gnome_wp_item_update_description (item);
-     
-    g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-    wp_props_load_wallpaper (item->filename, item, capplet);
-
-    gconf_client_set_string (capplet->client, WP_FILE_KEY,
-			     item->filename, NULL);
-    gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
-			     item->options, NULL);
-
-    path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-			      NULL, FALSE);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				  path, NULL, TRUE, 0.5, 0.0);
-    gtk_tree_path_free (path);
   } else {
-    gnome_wp_item_free (item);
+    item = gnome_wp_item_new (filename, capplet->wphash, capplet->thumbs);
+    if (item != NULL) {
+      wp_props_load_wallpaper (item->filename, item, capplet);
+    }
   }
+
+  return item;
 }
 
 static void gnome_wp_add_images (GnomeWPCapplet * capplet,
-				 GSList * images,
-				 gchar ** files) {
+				 GSList * images) {
   GdkCursor * cursor;
-  gint i;
+  GnomeWPItem * item;
 
+  item = NULL;
   cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
 				       GDK_WATCH);
   gdk_window_set_cursor (capplet->window->window, cursor);
@@ -162,34 +122,26 @@ static void gnome_wp_add_images (GnomeWPCapplet * capplet,
 
   if (images != NULL) {
     for (; images != NULL; images = images->next) {
-      gnome_wp_add_image (capplet, images->data);
-    }
-  } else {
-    for (i = 0; files && files[i]; i++) {
-      gnome_wp_add_image (capplet, files[i]);
+      item = gnome_wp_add_image (capplet, images->data);
     }
   }
 
   gdk_window_set_cursor (capplet->window->window, NULL);
+
+  if (item != NULL) {
+    gnome_wp_capplet_scroll_to_item (capplet, item);
+  }
 }
 
 static void gnome_wp_file_open_dialog (GtkWidget * widget,
 				       GnomeWPCapplet * capplet) {
-#if GTK_CHECK_VERSION (2, 3, 0)
   GSList * files;
-#else
-  gchar ** files;
-#endif
 
   switch (gtk_dialog_run (GTK_DIALOG (capplet->filesel))) {
   case GTK_RESPONSE_OK:
-#if GTK_CHECK_VERSION (2, 3, 0)
     files = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (capplet->filesel));
-    gnome_wp_add_images (capplet, files, NULL);
-#else
-    files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (capplet->filesel));
-    gnome_wp_add_images (capplet, NULL, files);
-#endif
+    gnome_wp_add_images (capplet, files);
+    g_slist_free (files);
   case GTK_RESPONSE_CANCEL:
   default:
     gtk_widget_hide (capplet->filesel);
@@ -197,44 +149,6 @@ static void gnome_wp_file_open_dialog (GtkWidget * widget,
   }
 }
 
-static void bg_add_multiple_files (GnomeVFSURI * uri,
-				   GnomeWPCapplet * capplet) {
-  GnomeWPItem * item;
-  GdkColor color1, color2;
-
-  item = g_hash_table_lookup (capplet->wphash, gnome_vfs_uri_get_path (uri));
-  if (item != NULL) {
-    return;
-  }
-
-  item = g_new0 (GnomeWPItem, 1);
-
-  item->filename = gnome_vfs_unescape_string_for_display (gnome_vfs_uri_get_path (uri));
-  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-  item->name = g_strdup (item->fileinfo->name);
-  item->options = gconf_client_get_string (capplet->client,
-					   WP_OPTIONS_KEY,
-					   NULL);
-
-  item->shade_type = gconf_client_get_string (capplet->client,
-					      WP_SHADING_KEY, NULL);
-  item->pri_color = gconf_client_get_string (capplet->client,
-					     WP_PCOLOR_KEY, NULL);
-  item->sec_color = gconf_client_get_string (capplet->client,
-					     WP_SCOLOR_KEY, NULL);
-
-  gdk_color_parse (item->pri_color, &color1);
-  gdk_color_parse (item->sec_color, &color2);
-     
-  item->pcolor = gdk_color_copy (&color1);
-  item->scolor = gdk_color_copy (&color2);
-
-  gnome_wp_item_update_description (item);
-  
-  g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-  wp_props_load_wallpaper (item->filename, item, capplet);
-}
-
 static void bg_properties_dragged_image (GtkWidget * widget,
 					 GdkDragContext * context,
 					 gint x, gint y,
@@ -244,77 +158,27 @@ static void bg_properties_dragged_image (GtkWidget * widget,
 
   if (info == TARGET_URI_LIST || info == TARGET_BGIMAGE) {
     GList * uris;
+    GSList * realuris = NULL;
 
     uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
 
     if (uris != NULL && uris->data != NULL) {
-      if (g_list_length (uris) == 1) {
-	GnomeVFSURI * uri = (GnomeVFSURI *) uris->data;
-	GnomeWPItem * item;
-	GtkTreePath * path;
+      GdkCursor * cursor;
 
-	item = g_hash_table_lookup (capplet->wphash,
-				    gnome_vfs_uri_get_path (uri));
+      cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
+					   GDK_WATCH);
+      gdk_window_set_cursor (capplet->window->window, cursor);
+      gdk_cursor_unref (cursor);
 
-	if (item == NULL) {
-	  GdkColor color1, color2;
-
-	  item = g_new0 (GnomeWPItem, 1);
-
-	  item->filename = gnome_vfs_unescape_string_for_display (gnome_vfs_uri_get_path (uri));
-	  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-	  item->name = g_strdup (item->fileinfo->name);
-	  item->options = gconf_client_get_string (capplet->client,
-						   WP_OPTIONS_KEY,
-						   NULL);
-	
-	  item->shade_type = gconf_client_get_string (capplet->client,
-						      WP_SHADING_KEY, NULL);
-	  item->pri_color = gconf_client_get_string (capplet->client,
-						     WP_PCOLOR_KEY, NULL);
-	  item->sec_color = gconf_client_get_string (capplet->client,
-						     WP_SCOLOR_KEY, NULL);
-
-	  gdk_color_parse (item->pri_color, &color1);
-	  gdk_color_parse (item->sec_color, &color2);
-     
-	  item->pcolor = gdk_color_copy (&color1);
-	  item->scolor = gdk_color_copy (&color2);
-
-	  gnome_wp_item_update_description (item);
-
-	  g_hash_table_insert (capplet->wphash, g_strdup (item->filename),
-			       item);
-	  wp_props_load_wallpaper (item->filename, item, capplet);
-	} else if (item->deleted) {
-	  item->deleted = FALSE;
-	  wp_props_load_wallpaper (item->filename, item, capplet);
-	}
-	gconf_client_set_string (capplet->client, WP_FILE_KEY,
-				 item->filename, NULL);
-	gconf_client_set_string (capplet->client, WP_OPTIONS_KEY,
-				 item->options, NULL);
-
-	path = gtk_tree_row_reference_get_path (item->rowref);
-	gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				  NULL, FALSE);
-	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				      path, NULL, TRUE, 0.5, 0.0);
-	gtk_tree_path_free (path);
-      } else if (g_list_length (uris) > 1) {
-	GdkCursor * cursor;
-
-	cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
-					     GDK_WATCH);
-	gdk_window_set_cursor (capplet->window->window, cursor);
-	gdk_cursor_unref (cursor);
-
-	g_list_foreach (uris, (GFunc) bg_add_multiple_files, capplet);
-
-	gdk_window_set_cursor (capplet->window->window, NULL);
+      for (; uris != NULL; uris = uris->next) {
+	realuris = g_slist_append (realuris,
+				   g_strdup (gnome_vfs_uri_get_path (uris->data)));
       }
+      gnome_wp_add_images (capplet, realuris);
+      gdk_window_set_cursor (capplet->window->window, NULL);
     }
     gnome_vfs_uri_list_free (uris);
+    g_slist_free (realuris);
   }
 }
 
@@ -713,6 +577,8 @@ static void gnome_wp_remove_wallpaper (GtkWidget * widget,
     item->deleted = TRUE;
 
     gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+
+    g_free (wpfile);
   }
   first = gtk_tree_path_new_first ();
   gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview),
@@ -724,7 +590,6 @@ static gboolean gnome_wp_load_stuffs (void * data) {
   GnomeWPCapplet * capplet = (GnomeWPCapplet *) data;
   gchar * imagepath, * style;
   GnomeWPItem * item;
-  GtkTreePath * path;
 
   style = gconf_client_get_string (capplet->client,
 				   WP_OPTIONS_KEY,
@@ -747,12 +612,7 @@ static gboolean gnome_wp_load_stuffs (void * data) {
       wp_props_load_wallpaper (item->filename, item, capplet);
     }
 
-    path = gtk_tree_row_reference_get_path (item->rowref);
-    gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-			      NULL, FALSE);
-    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				  path, NULL, TRUE, 0.5, 0.0);
-    gtk_tree_path_free (path);
+    gnome_wp_capplet_scroll_to_item (capplet, item);
 
     gnome_wp_option_menu_set (capplet, item->options, FALSE);
     gnome_wp_option_menu_set (capplet, item->shade_type, TRUE);
@@ -766,44 +626,11 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 				item->scolor->green,
 				item->scolor->blue, 65535);
   } else if (strcmp (style, "none") != 0) {
-    GdkColor color1, color2;
-
-    item = g_new0 (GnomeWPItem, 1);
-
-    item->filename = g_strdup (imagepath);
-
-    item->shade_type = gconf_client_get_string (capplet->client,
-						WP_SHADING_KEY, NULL);
-    item->pri_color = gconf_client_get_string (capplet->client,
-					       WP_PCOLOR_KEY, NULL);
-    item->sec_color = gconf_client_get_string (capplet->client,
-					       WP_SCOLOR_KEY, NULL);
-
-    gdk_color_parse (item->pri_color, &color1);
-    gdk_color_parse (item->sec_color, &color2);
-
-    item->pcolor = gdk_color_copy (&color1);
-    item->scolor = gdk_color_copy (&color2);
-
-    if (g_file_test (item->filename, G_FILE_TEST_EXISTS)) {
-      item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-      item->name = g_strdup (item->fileinfo->name);
-      item->options = gconf_client_get_string (capplet->client,
-					       WP_OPTIONS_KEY,
-					       NULL);
-      item->deleted = FALSE;
-
-      gnome_wp_item_update_description (item);
-
-      g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
+    item = gnome_wp_item_new (imagepath, capplet->wphash, capplet->thumbs);
+    if (item != NULL) {
       wp_props_load_wallpaper (item->filename, item, capplet);
 
-      path = gtk_tree_row_reference_get_path (item->rowref);
-      gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				NULL, FALSE);
-      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				    path, NULL, TRUE, 0.5, 0.0);
-      gtk_tree_path_free (path);
+      gnome_wp_capplet_scroll_to_item (capplet, item);
 
       gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (capplet->pc_picker),
 				  item->pcolor->red,
@@ -816,43 +643,15 @@ static gboolean gnome_wp_load_stuffs (void * data) {
 
       gnome_wp_option_menu_set (capplet, item->options, FALSE);
       gnome_wp_option_menu_set (capplet, item->shade_type, TRUE);
-    } else {
-      gnome_wp_item_free (item);
     }
   }
 
   item = g_hash_table_lookup (capplet->wphash, "(none)");
   if (item == NULL) {
-    GdkColor color1, color2;
-
-    item = g_new0 (GnomeWPItem, 1);
-
-    item->deleted = FALSE;
-    item->filename = g_strdup ("(none)");
-
-    item->shade_type = gconf_client_get_string (capplet->client,
-						WP_SHADING_KEY, NULL);
-    item->pri_color = gconf_client_get_string (capplet->client,
-					       WP_PCOLOR_KEY, NULL);
-    item->sec_color = gconf_client_get_string (capplet->client,
-					       WP_SCOLOR_KEY, NULL);
-
-    gdk_color_parse (item->pri_color, &color1);
-    gdk_color_parse (item->sec_color, &color2);
-
-    item->pcolor = gdk_color_copy (&color1);
-    item->scolor = gdk_color_copy (&color2);
-
-    item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-    item->name = g_strdup (item->fileinfo->name);
-    item->options = gconf_client_get_string (capplet->client,
-					     WP_OPTIONS_KEY,
-					     NULL);
-
-    gnome_wp_item_update_description (item);
-
-    g_hash_table_insert (capplet->wphash, g_strdup (item->filename), item);
-    wp_props_load_wallpaper (item->filename, item, capplet);
+    item = gnome_wp_item_new ("(none)", capplet->wphash, capplet->thumbs);
+    if (item != NULL) {
+      wp_props_load_wallpaper (item->filename, item, capplet);
+    }
   } else {
     if (item->deleted == TRUE) {
       item->deleted = FALSE;
@@ -860,12 +659,7 @@ static gboolean gnome_wp_load_stuffs (void * data) {
     }
 
     if (!strcmp (style, "none")) {
-      path = gtk_tree_row_reference_get_path (item->rowref);
-      gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				NULL, FALSE);
-      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				    path, NULL, TRUE, 0.5, 0.0);
-      gtk_tree_path_free (path);
+      gnome_wp_capplet_scroll_to_item (capplet, item);
     }
   }
   g_free (imagepath);
@@ -879,17 +673,25 @@ static gint gnome_wp_list_sort (GtkTreeModel * model,
 				GnomeWPCapplet * capplet) {
   gchar * foo, * bar;
   gchar * desca, * descb;
+  gint retval;
 
   gtk_tree_model_get (model, a, 1, &desca, 2, &foo, -1);
   gtk_tree_model_get (model, b, 1, &descb, 2, &bar, -1);
 
   if (!strcmp (foo, "(none)")) {
-    return -1;
+    retval =  -1;
   } else if (!strcmp (bar, "(none)")) {
-    return 1;
+    retval =  1;
   } else {
-    return g_utf8_collate (desca, descb);
+    retval = g_utf8_collate (desca, descb);
   }
+
+  g_free (desca);
+  g_free (descb);
+  g_free (foo);
+  g_free (bar);
+
+  return retval;
 }
 
 static void gnome_wp_file_changed (GConfClient * client, guint id,
@@ -898,7 +700,6 @@ static void gnome_wp_file_changed (GConfClient * client, guint id,
   GtkTreeSelection * selection;
   GtkTreeModel * model;
   GtkTreeIter iter;
-  GtkTreePath * path;
   GnomeWPItem * item;
   gchar * wpfile, * selected;
 
@@ -910,52 +711,13 @@ static void gnome_wp_file_changed (GConfClient * client, guint id,
     gtk_tree_model_get (model, &iter, 2, &selected, -1);
     if (strcmp (selected, wpfile) != 0) {
       if (item != NULL) {
-	path = gtk_tree_row_reference_get_path (item->rowref);
-	gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				  NULL, FALSE);
-	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-				      path, NULL, TRUE, 0.5, 0.0);
-	gtk_tree_path_free (path);
+	gnome_wp_capplet_scroll_to_item (capplet, item);
       } else {
-	GdkColor color1, color2;
-
-	item = g_new0 (GnomeWPItem, 1);
-
-	item->filename = g_strdup (wpfile);
-
-	item->shade_type = gconf_client_get_string (capplet->client,
-						    WP_SHADING_KEY, NULL);
-	item->pri_color = gconf_client_get_string (capplet->client,
-						   WP_PCOLOR_KEY, NULL);
-	item->sec_color = gconf_client_get_string (capplet->client,
-						   WP_SCOLOR_KEY, NULL);
-    
-	gdk_color_parse (item->pri_color, &color1);
-	gdk_color_parse (item->sec_color, &color2);
-    
-	item->pcolor = gdk_color_copy (&color1);
-	item->scolor = gdk_color_copy (&color2);
-    
-	if (g_file_test (item->filename, G_FILE_TEST_EXISTS)) {
-	  item->fileinfo = gnome_wp_info_new (item->filename, capplet->thumbs);
-	  item->name = g_strdup (item->fileinfo->name);
-	  item->options = gconf_client_get_string (capplet->client,
-						   WP_OPTIONS_KEY,
-						   NULL);
-      
-	  gnome_wp_item_update_description (item);
-	  g_hash_table_insert (capplet->wphash,
-			       g_strdup (item->filename), item);
+	item = gnome_wp_item_new (wpfile, capplet->wphash, capplet->thumbs);
+	if (item != NULL) {
 	  wp_props_load_wallpaper (item->filename, item, capplet);
       
-	  path = gtk_tree_row_reference_get_path (item->rowref);
-	  gtk_tree_view_set_cursor (GTK_TREE_VIEW (capplet->treeview), path,
-				    NULL, FALSE);
-	  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (capplet->treeview),
-					path, NULL, TRUE, 0.5, 0.0);
-	  gtk_tree_path_free (path);
-	} else {
-	  gnome_wp_item_free (item);
+	  gnome_wp_capplet_scroll_to_item (capplet, item);
 	}
       }
     }
@@ -1137,6 +899,16 @@ static GladeXML * gnome_wp_create_dialog (void) {
   return new;
 }
 
+static void set_accessible_name (GtkWidget *widget, const gchar *name) {
+  AtkObject *obj;
+ 
+  obj = gtk_widget_get_accessible (widget);
+  if (!GTK_IS_ACCESSIBLE (obj))
+    return;
+  if (name)
+    atk_object_set_name (obj, name);
+} 
+  
 static void wallpaper_properties_init (void) {
   GnomeWPCapplet * capplet;
   GladeXML * dialog;
@@ -1272,6 +1044,7 @@ static void wallpaper_properties_init (void) {
 
   menu = gtk_menu_new ();
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Centered")); 
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-center",
 					  16, NULL, NULL);
@@ -1295,6 +1068,7 @@ static void wallpaper_properties_init (void) {
   gtk_widget_show (mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Fill Screen"));
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-fill",
 					  16, NULL, NULL);
@@ -1317,6 +1091,7 @@ static void wallpaper_properties_init (void) {
   gtk_widget_show (mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Scaled"));
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-scale",
 					  16, NULL, NULL);
@@ -1337,6 +1112,7 @@ static void wallpaper_properties_init (void) {
   gtk_widget_show (mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Tiled"));
   icofile = gnome_icon_theme_lookup_icon (capplet->theme,
 					  "stock_wallpaper-tile",
 					  16, NULL, NULL);
@@ -1373,6 +1149,7 @@ static void wallpaper_properties_init (void) {
 
   menu = gtk_menu_new ();
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Solid Color"));
   mbox = gtk_hbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (mitem), mbox);
   gtk_widget_show (mbox);
@@ -1389,6 +1166,7 @@ static void wallpaper_properties_init (void) {
   gtk_menu_append (GTK_MENU (menu), mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Horizontal Gradient"));
   mbox = gtk_hbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (mitem), mbox);
   gtk_widget_show (mbox);
@@ -1405,6 +1183,7 @@ static void wallpaper_properties_init (void) {
   gtk_menu_append (GTK_MENU (menu), mitem);
 
   mitem = gtk_menu_item_new ();
+  set_accessible_name (mitem, _("Vertical Gradient"));
   mbox = gtk_hbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (mitem), mbox);
   gtk_widget_show (mbox);
@@ -1449,8 +1228,7 @@ static void wallpaper_properties_init (void) {
 		    G_CALLBACK (gnome_wp_props_wp_selected), capplet);
 
   /* Create the file chooser dialog stuff here */
-#if GTK_CHECK_VERSION (2, 3, 0)
-  capplet->filesel = gtk_file_chooser_dialog_new (_("Add Wallpapers"),
+  capplet->filesel = gtk_file_chooser_dialog_new (_("Add Wallpaper"),
 						  GTK_WINDOW (capplet->window),
 						  GTK_FILE_CHOOSER_ACTION_OPEN,
 						  GTK_STOCK_CANCEL,
@@ -1460,11 +1238,6 @@ static void wallpaper_properties_init (void) {
 						  NULL);
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (capplet->filesel),
 					TRUE);
-#else
-  capplet->filesel = gtk_file_selection_new (_("Add Wallpapers"));
-  gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (capplet->filesel),
-					  TRUE);
-#endif
 }
 
 gint main (gint argc, gchar *argv[]) {
